@@ -1,10 +1,12 @@
-#!/usr/bin/python
-# Copyright 1999-2007 Gentoo Foundation
+#!/usr/bin/python -b
+# Copyright 1999-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: filter-bash-environment.py 15539 2010-03-02 19:51:14Z zmedico $
 
 import codecs
-import os, re, sys
+import io
+import os
+import re
+import sys
 
 here_doc_re = re.compile(r'.*\s<<[-]?(\w+)$')
 func_start_re = re.compile(r'^[-\w]+\s*\(\)\s*$')
@@ -117,23 +119,30 @@ def filter_bash_environment(pattern, file_in, file_out):
 		file_out.write(line)
 
 if __name__ == "__main__":
-	description = "Filter out variable assignments for varable " + \
+	description = "Filter out variable assignments for variable " + \
 		"names matching a given PATTERN " + \
 		"while leaving bash function definitions and here-documents " + \
 		"intact. The PATTERN is a space separated list of variable names" + \
 		" and it supports python regular expression syntax."
 	usage = "usage: %s PATTERN" % os.path.basename(sys.argv[0])
-	from optparse import OptionParser
-	parser = OptionParser(description=description, usage=usage)
-	options, args = parser.parse_args(sys.argv[1:])
+	args = sys.argv[1:]
+
+	if '-h' in args or '--help' in args:
+		sys.stdout.write(usage + "\n")
+		sys.stdout.flush()
+		sys.exit(os.EX_OK)
+
 	if len(args) != 1:
-		parser.error("Missing required PATTERN argument.")
+		sys.stderr.write(usage + "\n")
+		sys.stderr.write("Exactly one PATTERN argument required.\n")
+		sys.stderr.flush()
+		sys.exit(2)
+
 	file_in = sys.stdin
 	file_out = sys.stdout
 	if sys.hexversion >= 0x3000000:
 		file_in = codecs.iterdecode(sys.stdin.buffer.raw,
 			'utf_8', errors='replace')
-		import io
 		file_out = io.TextIOWrapper(sys.stdout.buffer,
 			'utf_8', errors='backslashreplace')
 
